/**
   JWindowsScheduler - Java access library for the windows task scheduler
   Copyright (C) 2002 Steffen Siebert <siebert@SteffenSiebert.de>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package de.steffensiebert.jwindowsscheduler;

import java.util.Vector;
import java.lang.String;

public class WindowsScheduler {

  public Vector getTasks() throws OleException {
    Vector v = new Vector();
    int rc = Ole.CoInitialize();
    if (rc < 0) {
      throw new OleException("CoInitialize failed with returncode "+rc);
    }
    try {
      int taskScheduler = Ole.CoCreateInstance();
      int enum = Ole.Enum(taskScheduler);
      String taskName = null;
      while ((taskName = Ole.Next(enum)) != null) {
        v.add(taskName);
      }
      Ole.ReleaseEnum(enum);
      Ole.ReleaseTaskScheduler(taskScheduler);
      return v;
    } catch (OleException e) {
      throw e;
    } finally {
      Ole.CoUninitialize();
    }
  }

  public WindowsTask readTask(String taskName) throws OleException {
    WindowsTask t = new WindowsTask();
    int rc = Ole.CoInitialize();
    if (rc < 0) {
      throw new OleException("CoInitialize failed with returncode "+rc);
    }
    try {
      int taskScheduler = Ole.CoCreateInstance();
      int task = Ole.Activate(taskScheduler, taskName);
      Ole.ReleaseTaskScheduler(taskScheduler);
      t.taskName = taskName;
      t.application = Ole.GetApplicationName(task);
      t.parameter = Ole.GetParameters(task);
      t.workingDirectory = Ole.GetWorkingDirectory(task);
      t.priority = Ole.GetPriority(task);
      t.runTime = Ole.GetMaxRunTime(task);
      t.creator = Ole.GetCreator(task);
      t.comment = Ole.GetComment(task);
      t.flags = Ole.GetFlags(task);
      Ole.ReleaseTask(task);
      return t;
    } catch (OleException e) {
      throw e;
    } finally {
      Ole.CoUninitialize();
    }
  }

  public void writeTask(WindowsTask t) throws OleException {
    int rc = Ole.CoInitialize();
    if (rc < 0) {
      throw new OleException("CoInitialize failed with returncode "+rc);
    }
    int taskScheduler = 0;
    try {
      taskScheduler = Ole.CoCreateInstance();
      int task = Ole.NewWorkItem(taskScheduler, t.taskName);
      Ole.SetApplicationName(task,t.application);
      Ole.SetParameters(task,t.parameter);
      Ole.SetWorkingDirectory(task,t.workingDirectory);
      Ole.SetPriority(task,t.priority);
      Ole.SetMaxRunTime(task,t.runTime);
      Ole.SetComment(task,t.comment);
      Ole.SetFlags(task,t.flags);
      // Bei Windows 9x Systemen darf SetAccountInformation() nicht
      // aufgerufen werden. Wir erkennen dies daran, dass der
      // accountName nicht gesetzt ist. Das aufrufende Programm
      // muss daher ueberpruefen, auf welchem System es laeuft
      // und den accountName entsprechend setzen oder nicht!
      if (t.accountName != null) {
        Ole.SetAccountInformation(task,t.accountName,t.password);
      }
      int persistFile = Ole.QueryInterface(task);
      Ole.ReleaseTask(task);
      Ole.Save(persistFile);
      return;
    } catch (OleException e) {
      throw e;
    } finally {
      if (taskScheduler != 0) {
        Ole.ReleaseTaskScheduler(taskScheduler);
      }
      Ole.CoUninitialize();
    }
  }

  public static void main(String[] args) {
    WindowsScheduler s = new WindowsScheduler();
//     WindowsTask t = new WindowsTask();
//     t.setTaskName("Test Task");
//     t.setApplication("WinTV2K.EXE");
//     t.setParameter(" -c9 -ntod -startr:test###.mpg -qvcd -limit:7440");
//     t.setWorkingDirectory("");
//     t.setPriority(32);
//     t.setComment("HCW Scheduler");
//     t.setRunTime(-1);
//     s.writeTask(t);
//     WindowsTaskTrigger tr = new WindowsTaskTriggerOnce(12,7,2002,21,12);
//     t.writeTrigger(tr);
//     tr = new WindowsTaskTriggerOnce(12,8,2002,21,12);
//     t.writeTrigger(tr);
    Vector v = s.getTasks();
    for (int i = 0; i < v.size(); i++) {
      String taskName = (String) v.elementAt(i);
      System.out.println(taskName);
      WindowsTask task = s.readTask(taskName);
      System.out.println(task);
      System.out.println(task.getTriggers());
    }
  }
}
