/**
   JWindowsScheduler - Java access library for the windows task scheduler
   Copyright (C) 2002 Steffen Siebert <siebert@SteffenSiebert.de>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package de.steffensiebert.jwindowsscheduler;

import java.util.Vector;
import java.lang.String;

public class WindowsTask {
  public static int TASK_FLAG_DELETE_WHEN_DONE = 2;
  public static int TASK_FLAG_RUN_ONLY_IF_LOGGED_ON = 8192;

  String taskName = null;
  String application = null;
  String parameter = null;
  String workingDirectory = null;
  int priority = 0;
  int runTime = 0;
  String creator = null;
  String comment = null;
  int flags = 0;
  String accountName = null;
  String password = null;

  public Vector getTriggerStrings() {
    int rc = Ole.CoInitialize();
    if (rc < 0) {
      // FIXME
      return null;
    }
    try {
      int taskScheduler = Ole.CoCreateInstance();
      int task = Ole.Activate(taskScheduler, taskName);
      Ole.ReleaseTaskScheduler(taskScheduler);
      int triggerCount = Ole.GetTriggerCount(task);
      System.out.println("triggerCount: "+triggerCount);
      Vector triggers = new Vector();
      for (int i=0; i<triggerCount; i++) {
        triggers.add(Ole.GetTriggerString(task, i));
      }
      Ole.ReleaseTask(task);
      return triggers;
    } catch (Exception e) {
      // FIXME
      e.printStackTrace();
    } finally {
      Ole.CoUninitialize();
    }
    return null;
  }

  public Vector getTriggers() {
    int rc = Ole.CoInitialize();
    if (rc < 0) {
      // FIXME
      return null;
    }
    try {
      int taskScheduler = Ole.CoCreateInstance();
      int task = Ole.Activate(taskScheduler, taskName);
      Ole.ReleaseTaskScheduler(taskScheduler);
      int triggerCount = Ole.GetTriggerCount(task);
      System.out.println("triggerCount: "+triggerCount);
      Vector triggers = new Vector();
      int[] values = new int[18];
      for (int i=0; i<triggerCount; i++) {
        WindowsTaskTrigger t = null;
        int triggerType = Ole.GetTriggerType(task, i);
        if (WindowsTaskTrigger.TASK_TIME_TRIGGER_ONCE == triggerType) {
          t = new WindowsTaskTriggerOnce();
        } else if (WindowsTaskTrigger.TASK_TIME_TRIGGER_DAILY == triggerType) {
          t = new WindowsTaskTriggerDaily();
        } else if (WindowsTaskTrigger.TASK_TIME_TRIGGER_WEEKLY == triggerType) {
          t = new WindowsTaskTriggerWeekly();
        } else if (WindowsTaskTrigger.TASK_TIME_TRIGGER_MONTHLYDATE == triggerType) {
          t = new WindowsTaskTriggerMonthlyDate();
        } else if (WindowsTaskTrigger.TASK_TIME_TRIGGER_MONTHLYDOW == triggerType) {
          t = new WindowsTaskTriggerMonthlyDOW();
        } else if (WindowsTaskTrigger.TASK_TIME_TRIGGER_ON_IDLE == triggerType) {
          t = new WindowsTaskTriggerOnIdle();
        } else if (WindowsTaskTrigger.TASK_TIME_TRIGGER_AT_SYSTEMSTART == triggerType) {
          t = new WindowsTaskTriggerAtSystemstart();
        } else if (WindowsTaskTrigger.TASK_TIME_TRIGGER_AT_LOGON == triggerType) {
          t = new WindowsTaskTriggerAtLogon();
        } else {
          // FIXME
          return null;
        }
        Ole.GetTriggerValues(task, i, values);
        t.beginYear = values[0];
        t.beginMonth = values[1];
        t.beginDay = values[2];
        t.startHour = values[3];
        t.startMinute = values[4];
        t.endYear = values[5];
        t.endMonth = values[6];
        t.endDay = values[7];
        t.minutesDuration = values[8];
        t.minutesInterval = values[9];
        t.rgFlags = values[10];
        t.triggerType = values[11];
        t.daysInterval = values[12];
        t.weeksInterval = values[13];
        t.rgfDaysOfTheWeek = values[14];
        t.rgfDays = values[15];
        t.rgfMonths = values[16];
        t.whichWeek = values[17];
        triggers.add(t);
      }
      Ole.ReleaseTask(task);
      return triggers;
    } catch (Exception e) {
      // FIXME
      e.printStackTrace();
    } finally {
      Ole.CoUninitialize();
    }
    return null;
  }

  public void writeTrigger(WindowsTaskTrigger t) {
    int rc = Ole.CoInitialize();
    if (rc < 0) {
      // FIXME
      return;
    }
    int[] values = new int[18];
    values[0] = t.beginYear;
    values[1] = t.beginMonth;
    values[2] = t.beginDay;
    values[3] = t.startHour;
    values[4] = t.startMinute;
    values[5] = t.endYear;
    values[6] = t.endMonth;
    values[7] = t.endDay;
    values[8] = t.minutesDuration;
    values[9] = t.minutesInterval;
    values[10] = t.rgFlags;
    values[11] = t.triggerType;
    values[12] = t.daysInterval;
    values[13] = t.weeksInterval;
    values[14] = t.rgfDaysOfTheWeek;
    values[15] = t.rgfDays;
    values[16] = t.rgfMonths;
    values[17] = t.whichWeek;
    try {
      int taskScheduler = Ole.CoCreateInstance();
      int task = Ole.Activate(taskScheduler, taskName);
      Ole.ReleaseTaskScheduler(taskScheduler);
      int trigger = Ole.CreateTrigger(task);
      Ole.SetTrigger(trigger,values);
      int persistFile = Ole.QueryInterface(task);
      Ole.ReleaseTask(task);
      Ole.Save(persistFile);
    } catch (OleException e) {
      // FIXME
      e.printStackTrace();
    } finally {
      Ole.CoUninitialize();
    }
    return;
  }

  public void setTaskName(String taskName) {
    this.taskName = taskName;
  }

  public String getTaskName() {
    return taskName;
  }

  public void setApplication(String application) {
    this.application = application;
  }

  public void setParameter(String parameter) {
    this.parameter = parameter;
  }

  public void setWorkingDirectory(String workingDirectory) {
    this.workingDirectory = workingDirectory;
  }

  public void setPriority(int priority) {
    this.priority = priority;
  }

  public void setRunTime(int runTime) {
    this.runTime = runTime;
  }

  public String getComment() {
    return comment;
  }

  public void setComment(String comment) {
    this.comment = comment;
  }

  public void setFlags(int flags) {
    this.flags = flags;
  }

  public void setAccountName(String accountName) {
    this.accountName = accountName;
  }

  public void setPassword(String password) {
    this.password = password;
  }

  public String toString() {
    String s = "Dies ist ein WindowsTask Objekt!\n";
    s = s + "taskName: <"+taskName+">\n";
    s = s + "application: <"+application+">\n";
    s = s + "parameter: <"+parameter+">\n";
    s = s + "workingDirectory: <"+workingDirectory+">\n";
    s = s + "priority: <"+priority+">\n";
    s = s + "runTime: <"+runTime+">\n";
    s = s + "creator: <"+creator+">\n";
    s = s + "comment: <"+comment+">\n";
    s = s + "flags: <"+flags+">\n";
    return s;
  }
}
